// Require jQuery
require(['jquery', 'xwiki-meta'], function($, xm) {
  'use strict';

  // Waiting jQuery to be ready
  $(document).ready(function() {
  
    // Usefull variables
    var iframe = $("#iframe");
    var autoSync = $("#autosync");
    var variableInputs = $("#bt-variables input,#bt-variables select");
    
    // Need IE9+ or any other recent browser
    var match = navigator.userAgent.match(/(?:MSIE |Trident\/.*; rv:)(\d+)/);
    if (match && parseInt(match[1]) < 9) {
      iframe.parent().replaceWith("<div class=\"panel-body\"><div class=\"alert alert-danger\">Sorry, the preview does not work with this version of Internet Explorer. You need to use a more recent browser.</div></div>");
      return;
    }
    
    // Put a gray curtain in front of the iframe
    var addCurtain = function() {
      var previewCurtain = $("#preview-curtain");
      if (previewCurtain.length > 0) {
        // The curtain already exist, do nothing.
        return;
      }
      var iframePosition = iframe.position();
      previewCurtain = $("<div id=\"preview-curtain\"><h2>Loading...</h2></div>");
      previewCurtain.insertAfter(iframe);
      previewCurtain.width(iframe.outerWidth(true));
      previewCurtain.height(iframe.outerHeight(true));
      previewCurtain.css("left", iframePosition.left);
      previewCurtain.css("right", iframePosition.right);
    }
    
    // Remove the curtain
    var removeCurtain = function() {
      $("#preview-curtain").remove();
    }
    
    // Function that waits the iframe to be fully loaded
    var waitUntilIFrameIsLoaded = function(callback) {
      var iframeHead = $(iframe).contents().find("head")[0];
      if (iframeHead !== undefined) {
        callback();
      } else {
        setTimeout(function(){ waitUntilIFrameIsLoaded(callback); }, 100);
      }
    };
    
    // Function that waits the iframe has loaded LESS
    var waitUntilLessIsLoadedInIFrame = function(callback) {
      if (iframe[0].contentWindow.lessJs !== undefined) {
        callback();
      } else {
        setTimeout(function(){ waitUntilLessIsLoadedInIFrame(callback); }, 100 );
      }
    }
    
    // Refresh the preview
    var refreshPreview = function () {
      addCurtain();
      
      // Set the good logo when the iframe is refreshed
      setLogo();
      
      // Get the variables' values
      var varMap = {};
      for (var i=0; i<variableInputs.length; ++i) {
        var variableInput = variableInputs[i];
        // 'logo' is handled by the 'setLogo' function
        // 'lessCode' is handled afterwards
        if (variableInput.value != '' && !variableInput.name.match("_0_logo$") && variableInput.id != 'var-lessCode') {
          var name = '@' + variableInput.id.substr(4); // we remove the 'var-' part of the id to get the real variable name
          var value = variableInput.value
          var input = $(variableInput);
          // Some values need to be placed inside a string
          if (input.hasClass('type-image')) {
            var url = $("#attachmentsURL").attr('data-value');
            value = 'url('+ url.replace("__tochange__", value) + ')';
          }
          if (input.hasClass('type-escapedText')) {
            value = '"' + value.replace('"', '\\"') + '"';
          }
          varMap[name] = value;
        }
      }
      
      // 'lessCode' is a special case: it is not a LESS variable, but it contains some code to inject in the LESS file.
      // It is handled by our LessCodePlugin.
      iframe[0].contentWindow.lessCodePlugin.contentToAdd = $('#var-lessCode').val();
      
      // We run LESS in a thread in order to not block the main thread
      setTimeout(function(){
        // Run LESS (which use promises)
        // Note: because of yuicompressor which does not support Promise.prototype.catch() properly, we cannot
        // call p.catch() directly (it generates an error in the logs).
        // See https://jira.xwiki.org/browse/XWIKI-13772
        // Instead, we use the following trick: p['catch']()
        // TODO: remove this trick when we replace yuicompressor by something else.
        iframe[0].contentWindow.lessJs.modifyVars(varMap).then(removeCurtain)['catch'](removeCurtain);
      }, 100);
    };
    
    // Function called when an input is changed by the user
    var onChange = function () {
      var input = $(this);
      if (autoSync.prop("checked") && input.closest('.type-image').length == 0) {
        var oldValue = variableInput.value;
        setTimeout(function(){
          if (variableInput.value == oldValue) {
            refreshPreview();
          }
        }, 1000);
      }
    }
    
    // Function that change the logo in the iframe
    var setLogo = function(event) {
      var value = $("[name$='_logo']").val();
      var iFrameLogo = iframe.contents().find("#companylogo > a > img");
      if (value == "") {
        var url = "../../../skins/flamingo/logo.svg";
        iFrameLogo.attr("src", url);
      } else {
        var url = $("#attachmentsURL").attr('data-value');
        iFrameLogo.attr("src", url.replace("__tochange__", value));
      }
    }
    
    // Function that adds LESS scripts to the iframe document
    var initIframe = function() {
      addCurtain();
      var iframeHead = iframe.contents().find("head")[0];
      
      // 1: Add a link to the LESS source code of the skin
      var link = document.createElement("link");
      link.rel = "stylesheet/less";
      link.type = "text/css";
      link.href = "../../../skins/flamingo/less/style.less.vm?colorTheme=no";
      iframeHead.appendChild(link);
      
      // 2: Add the LESS plugin we have developed which appends dynamically the content of the @lessCode variable
      // to the LESS code of the skin.
      
      // The plugin is an AMD module defined in an other page (we add a query string so require.js does not append the '.js' suffix).
      var pluginURL = new XWiki.Document(new XWiki.DocumentReference(xm.wiki, ['FlamingoThemesCode'], 'LessCodePluginJs')).getURL('jsx', 'r=1');
      
      // We add a script tag to the iframe that will load our plugin and the LESS preprocessor
      var script = document.createElement("script");
      script.innerHTML  = "\
        require.config({ \n\
          paths: { \n\
            'lessJs': '../../../webjars/less/2.5.3/less.min.js', \n\
            'lessCodePlugin': '" + pluginURL + "' \n\
          } \n\
        }); \n\
        // Now we load the LessCodePlugin BEFORE loading the LESS prepocessor. \n\
        // This is needed because the 'less' variable defining the list of enabled plugins \n\
        // must be declared *before* the LESS preprocessor. \n\
        // See: http://lesscss.org/usage/#plugins-in-the-browser \n\
        require(['lessCodePlugin'], function(lessCodePlugin) { \n\
          // We defined it as a global variable so we can re-use it afterwards.\n\
          window.lessCodePlugin = lessCodePlugin; \n\
          less = { plugins: [lessCodePlugin] }; \n\
          // Now we can load the LESS preprocessor. \n\
          require(['lessJs'], function(lessJs) { \n\
            // We defined it as a global variable so we can re-use it afterwards.\n\
            window.lessJs = lessJs; \n\
          }); \n\
        });";
      iframeHead.appendChild(script);
      
      // 3: Refresh the live preview
      waitUntilLessIsLoadedInIFrame(function(){
        // We need to refresh just after LESS is loaded because the fist rendering does not handle the variables of the
        // flamingo theme. Unfortunately, it means that the preview is rendered twice at the loading of the page.
        // We refresh it in a thread to avoid blocking the browser.
        setTimeout(refreshPreview, 1);
      });
    };
    
    // Add the curtain which will be removed when the iframe will be loaded and the LESS code compiled
    addCurtain()
    // The user can navigate to an other page inside the iframe, so when the page changes (onload),
    // we re-init the iframe to use LESS
    iframe.load(initIframe);
    // Initialize the event handlers
    $("#refresh").click( function() { 
      // We refresh it in a thread to avoid blocking the browser.
      setTimeout(refreshPreview, 1);
    });
    for (var i=0; i<variableInputs.length; ++i) {
      var variableInput = variableInputs[i];
      $(variableInput).on("change", onChange);
    }
    // Event handler for the logo field
    var logoFieldInput = $("#var-logo input");
    logoFieldInput.on("change", setLogo);

    // Since the input is hidden (by the attachmentSelector macro), we need to do some black magic (i.e. register a MutationObserver) to make it trigger the "change" event.
    MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
    var trackChange = function(element) {
      var observer = new MutationObserver(function(mutations, observer) {
        if (mutations[0].attributeName == "value") {
          $(element).trigger("change");
        }
      });
      observer.observe(element, {
        attributes: true
      });
    }
    // Enable change tracking for the logo field.
    trackChange(logoFieldInput[0]);
  });
});

// Configure require.js to use Colpick (https://github.com/josedvq/colpick-jQuery-Color-Picker/)
require.config({
  paths: {
    'colpick': "../../../webjars/colpick/2.0.2/js/colpick"
  },
  shim: {
    'colpick' : ['jquery']
  }
});
// Beginning of the script
require(['jquery', 'colpick'], function($, Colpick) {

  // Get the inputs that need to be enhanced
  var inputs = $('.color-picker');
  
  // For each input
  for (var i=0; i<inputs.length; ++i) {
    var input = $(inputs[i]);
    var parent = input.parent();
    
    // Create the color preview box
    parent.append('<span class="input-group-addon"><span class="color-preview"></span></span>');
    
    // Update the color preview box
    parent.find('.color-preview').css('background-color', input.val());
    
    // Enable the color picker
    input.colpick({
      layout: 'hex',
      submit: true,
      color: input.val(),
      onSubmit: function(hsb, hex, rgb, el) {
        var element = $(el);
        // Hide the color picker
        element.colpickHide();
        // Update the filled value
        element.val('#'+hex);
        // Update the color preview box
        element.parent().find('.color-preview').css('background-color', '#'+hex);
        // Emit the "change" event
        element.trigger('change');
      }
    });
    
    // If the user write a value manually
    input.keyup(function(event){
      var element = $(event.target);
      // Update the color preview box
      element.parent().find('.color-preview').css('background-color', element.val());
      // Update the color picker
      element.colpickSetColor(element.val(), false);
    });
  }
  
  // Hide the color picker if the user scrolls
  $('.tab-content').scroll(function(){
    // For each input
    for (var i=0; i<inputs.length; ++i) {
      var input = inputs[i];
      $(input).colpickHide();
    }
  });
});

